package IO::Capture::Stderr::Extended;
use strict;
use warnings;
use base 'IO::Capture::Stderr';
use IO::Capture::Extended qw(:all);

1;

########## DOCUMENTATION ##########

=head1 NAME

IO::Capture::Stderr::Extended - Extend functionality of IO::Capture::Stderr

=head1 SYNOPSIS

All documentation is contained in IO::Capture::Extended::Overview.

    perldoc IO::Capture::Extended::Overview

=cut

